function ends = endsalgour(purezerosinsp, purezerosexp, flow, co2)



len = min(length(purezerosinsp), length(purezerosexp));

breathendsindices = [1:len];

printf("\n......  OUR algorithm starts  ..............................................\n");
printf("Zero crossings before filtering: %d\n", len);

% =========================================================================================
% 1) find peak corresponding to each purezerosinsp (end of zerocrossings) 
% =========================================================================================
co2peakindices = [];

for i=1:length(breathendsindices)

  co2peakindex = findpeak(purezerosinsp(breathendsindices(i)), co2);
  co2peakindices = [co2peakindices; co2peakindex];

end


% brát nejbližší?
[purezerosinsp - co2peakindices];


%co2(co2peakindices)

% =========================================================================================
% 2) filter out purezeros corresponding to co2 peak with low concentration  
% =========================================================================================

newindices = [];

for i=1:len
  
  co2peakindex = co2peakindices(i);
  if (co2( co2peakindex ) > 2)
    newindices = [newindices; i];  
  end

end

breathendsindices = newindices;
printf("Zero crossings after filtering low CO2: %d\n", length(breathendsindices));



% =========================================================================================
% ) compute volumes of all phases and filter out all breath with 
% Vout/Vin > 5
% =========================================================================================

Vout = [];
Vin = [];

for breathnum=1:(len-1)

  sumout = 0;
  sumin = 0;
  
  exps = purezerosexp(breathnum); 
  expe = purezerosinsp(breathnum);
  for i=exps:(expe-1)
    sumout = sumout + 0.0025*(flow(i) + flow(i+1)); % 0.0025 = 0.005*0.5
  end

  insps = purezerosinsp(breathnum);
  inspe = purezerosexp(breathnum+1);
  for i=insps:(inspe-1)
    sumin = sumin + 0.0025*(flow(i) + flow(i+1));
  end


  Vout = [Vout; -sumout];
  Vin = [Vin; sumin];

end
 
 
 treshold = 5;
 %Vout ./ Vin
 Vratio = (Vout ./ Vin) >  treshold;
 Vratio = [Vratio; 0];




% =========================================================================================
% 3) filter out CO2 peaks where concentration between two consecutive peaks 
% does not drop below a certain level  
% =========================================================================================

newindices = [];
CO2min = [];
len = length(breathendsindices);

for i =1:len-1
  s = purezerosinsp(breathendsindices(i));
  e = purezerosinsp(breathendsindices(i+1));
  CO2min = [CO2min; min(co2(s:e))];
end


CO2min = CO2min > 0.5;
CO2min = [CO2min; 0]; % stopping criterion for next filters


i = 1;
while i <= len

   if (CO2min(i)) 
      j = i + 1;
      while j <= len && CO2min(j) 
         j = j + 1;       
      end
      newindices = [newindices; infsup(breathendsindices(i), breathendsindices(j))];  
      i = j + 1; 
   else
      newindices = [newindices; infsup(breathendsindices(i))];
      i = i + 1;   
   end

end

breathendsindices = newindices;
printf("Zerocrossings after filtering insuficient drops of CO2: %d\n", length(breathendsindices));
  
 
% =========================================================================================
% 4 )  filter out all breaths with 
% Vout/Vin > 5
% ========================================================================================= 
 
len = length(breathendsindices);
newindices = []; 

i = 1;
while i <= len

   if (Vratio(sup(breathendsindices(i)))) 
      j = i + 1;
      while (j <= len) && (inf(breathendsindices(j)) == (sup(breathendsindices(j-1)) + 1) ) &&  Vratio(sup(breathendsindices(j-1))) 
         j = j + 1;       
      end
      newindices = [newindices; infsup(inf(breathendsindices(i)), sup(breathendsindices(j-1)))];  
      i = j; 
   else
      newindices = [newindices; breathendsindices(i)];
      i = i + 1;   
   end

end

breathendsindices = newindices;




printf("Zerocrossings after filtering according to in/out Volume: %d\n", length(breathendsindices));


ends = infsup( purezerosinsp(inf(breathendsindices)), purezerosinsp(sup(breathendsindices)) );
 
 
 

 

%spaces = purezerosinsp(2:len) - purezerosinsp(1:len-1) ; 
%m = mean(spaces);  % jak nastavit ??????? na fyziologickém základě?????
%sd = std(spaces);
%
%printf("\nMean index space between two breaths: %d \n", m);      
%printf("\nMean time space between two breaths: %d s \n", m*0.005);      
%printf("Standard deviation: %d s \n", sd*0.005);  
%printf("Minimum distance: %d s \n", min(spaces)*0.005); 
%printf("Maximum distance: %d s \n", max(spaces)*0.005); 




end

